package il.ac.tau.cs.sw1.sortedset;

/**
 * A Set that provides a total ordering on its elements. The elements are 
 * ordered using their natural ordering. All elements inserted into a 
 * sorted set must implement the Comparable interface. 
 * The set's iterator will traverse the set in ascending element order.
 */
public interface SortedSet<T extends Comparable<T>> {
	
	/**
	 * Return <code>true</code> if this set contains the specified element.
	 * @return <code>true</code> if this set contains the specified element 
	 */
	public boolean contains (T element);

	/**
	 * Return the maximal (highest) element in this set.
	 * @return the maximal (highest) element in this set 
	 * @throws NoSuchElementException - if this set is empty
	 */
	public T getMaximum();
	
	/**
	 * Return the minimal (lowest) element in this set.
	 * @return the minimal (lowest) element in this set 
	 * @throws NoSuchElementException - if this set is empty
	 */
	public T getMinimum();
	
	/**
	 * Return <code>true</code> if this set contains no elements.
	 * @return <code>true</code> if this set contains no elements
	 */
	public boolean isEmpty ();
	
	/**
	 * Return an iterator over the elements of this set. The 
	 * elements are returned in ascending order. 
	 * @return	an iterator over the elements in this set
	 */
	public SortedIterator<T> iterator();
	
	/**
	 * Return the number of elements in this set (its cardinality).
	 * @return the number of elements in this set (its cardinality)
	 */
	public int size();
	
}
